@@ -1,48 +1,117 @@
-/****************************************************************************
-**
-** Copyright (C) 2019 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of Qt Creator.
-**
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 as published by the Free Software
-** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-****************************************************************************/
+// Copyright (C) 2019 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
 
 #include "compilationdbparser.h"
 
+#include "compilationdatabaseconstants.h"
+#include "compilationdatabaseprojectmanagertr.h"
+
 #include <coreplugin/progressmanager/progressmanager.h>
-#include <projectexplorer/task.h>
+
+#include <extensionsystem/pluginmanager.h>
+
 #include <projectexplorer/treescanner.h>
-#include <utils/mimetypes/mimetype.h>
-#include <utils/runextensions.h>
+
+#include <utils/async.h>
+#include <utils/futuresynchronizer.h>
+#include <utils/mimeutils.h>
 
 #include <QCryptographicHash>
-#include <QDir>
-#include <QFileInfo>
 #include <QJsonArray>
 #include <QJsonDocument>
 #include <QJsonObject>
 
+#include <vector>
+
 using namespace ProjectExplorer;
 using namespace Utils;
 
-namespace CompilationDatabaseProjectManager {
-namespace Internal {
+namespace CompilationDatabaseProjectManager::Internal {
+
+static QStringList jsonObjectFlags(const QJsonObject &object, QSet<QString> &flagsCache)
+{
+    QStringList flags;
+    const QJsonArray arguments = object["arguments"].toArray();
+    if (arguments.isEmpty()) {
+        flags = splitCommandLine(object["command"].toString(), flagsCache);
+    } else {
+        flags.reserve(arguments.size());
+        for (const QJsonValue &arg : arguments) {
+            auto flagIt = flagsCache.insert(arg.toString());
+            flags.append(*flagIt);
+        }
+    }
+    return flags;
+}
+
+static FilePath jsonObjectFilePath(const QJsonObject &object)
+{
+    const FilePath workingDir = FilePath::fromUserInput(object["directory"].toString());
+    return workingDir.resolvePath(object["file"].toString());
+}
+
+static std::vector<DbEntry> readJsonObjects(const QByteArray &projectFileContents)
+{
+    std::vector<DbEntry> result;
+
+    int objectStart = projectFileContents.indexOf('{');
+    int objectEnd = projectFileContents.indexOf('}', objectStart + 1);
+
+    QSet<QString> flagsCache;
+    while (objectStart >= 0 && objectEnd >= 0) {
+        const QJsonDocument document = QJsonDocument::fromJson(
+            projectFileContents.mid(objectStart, objectEnd - objectStart + 1));
+        if (document.isNull()) {
+            // The end was found incorrectly, search for the next one.
+            objectEnd = projectFileContents.indexOf('}', objectEnd + 1);
+            continue;
+        }
+
+        const QJsonObject object = document.object();
+        const FilePath filePath = jsonObjectFilePath(object);
+        const QStringList flags = filterFromFileName(jsonObjectFlags(object, flagsCache),
+                                                     filePath.fileName());
+        result.push_back({flags, filePath, FilePath::fromUserInput(object["directory"].toString())});
+
+        objectStart = projectFileContents.indexOf('{', objectEnd + 1);
+        objectEnd = projectFileContents.indexOf('}', objectStart + 1);
+    }
+    return result;
+}
+
+static QStringList readExtraFiles(const QString &filePath)
+{
+    QStringList result;
+
+    QFile file(filePath);
+    if (file.open(QFile::ReadOnly)) {
+        QTextStream stream(&file);
+        while (!stream.atEnd()) {
+            const QString line = stream.readLine().trimmed();
+            if (line.isEmpty() || line.startsWith('#'))
+                continue;
+
+            result.push_back(line);
+        }
+    }
+    return result;
+}
+
+static DbContents parseProject(const QByteArray &projectFileContents,
+                               const FilePath &projectFilePath)
+{
+    DbContents dbContents;
+    dbContents.entries = readJsonObjects(projectFileContents);
+    dbContents.extraFileName = projectFilePath.toString() +
+                               Constants::COMPILATIONDATABASEPROJECT_FILES_SUFFIX;
+    dbContents.extras = readExtraFiles(dbContents.extraFileName);
+    std::sort(dbContents.entries.begin(), dbContents.entries.end(),
+              [](const DbEntry &lhs, const DbEntry &rhs) {
+                  return std::lexicographical_compare(lhs.flags.begin(), lhs.flags.end(),
+                                                      rhs.flags.begin(), rhs.flags.end());
+              });
+    return dbContents;
+}
 
 CompilationDbParser::CompilationDbParser(const QString &projectName,
                                          const FilePath &projectPath,
@@ -59,8 +128,7 @@ CompilationDbParser::CompilationDbParser(const QString &projectName,
 {
     connect(&m_parserWatcher, &QFutureWatcher<void>::finished, this, [this] {
         m_dbContents = m_parserWatcher.result();
-        if (!m_treeScanner || m_treeScanner->isFinished())
-            finish(ParseResult::Success);
+        parserJobFinished();
     });
 }
 
@@ -80,6 +148,7 @@ void CompilationDbParser::start()
         return;
     }
     m_projectFileHash = newHash;
+    m_runningParserJobs = 0;
 
     // Thread 1: Scan disk.
     if (!m_rootPath.isEmpty()) {
@@ -102,25 +171,26 @@ void CompilationDbParser::start()
 
             return isIgnored;
         });
-        m_treeScanner->setTypeFactory([](const Utils::MimeType &mimeType, const Utils::FilePath &fn) {
+        m_treeScanner->setTypeFactory([](const MimeType &mimeType, const FilePath &fn) {
             return TreeScanner::genericFileType(mimeType, fn);
         });
         m_treeScanner->asyncScanForFiles(m_rootPath);
         Core::ProgressManager::addTask(m_treeScanner->future(),
-                                       tr("Scan \"%1\" project tree").arg(m_projectName),
+                                       Tr::tr("Scan \"%1\" project tree").arg(m_projectName),
                                        "CompilationDatabase.Scan.Tree");
-        connect(m_treeScanner, &TreeScanner::finished, this, [this] {
-            if (m_parserWatcher.isFinished())
-                finish(ParseResult::Success);
-        });
+        ++m_runningParserJobs;
+        connect(m_treeScanner, &TreeScanner::finished,
+                this, &CompilationDbParser::parserJobFinished);
     }
 
     // Thread 2: Parse the project file.
-    const QFuture<DbContents> future = runAsync(&CompilationDbParser::parseProject, this);
+    const auto future = Utils::asyncRun(parseProject, m_projectFileContents, m_projectFilePath);
     Core::ProgressManager::addTask(future,
-                                   tr("Parse \"%1\" project").arg(m_projectName),
+                                   Tr::tr("Parse \"%1\" project").arg(m_projectName),
                                    "CompilationDatabase.Parse");
+    ++m_runningParserJobs;
     m_parserWatcher.setFuture(future);
+    ExtensionSystem::PluginManager::futureSynchronizer()->addFuture(future);
 }
 
 void CompilationDbParser::stop()
@@ -138,109 +208,24 @@ void CompilationDbParser::stop()
 
 QList<FileNode *> CompilationDbParser::scannedFiles() const
 {
+    const bool canceled = m_treeScanner->future().isCanceled();
     const TreeScanner::Result result = m_treeScanner->release();
-    return m_treeScanner && !m_treeScanner->future().isCanceled() ? result.allFiles
-                                                                  : QList<FileNode *>();
+    return !canceled ? result.allFiles : QList<FileNode *>();
+}
+
+void CompilationDbParser::parserJobFinished()
+{
+    if (--m_runningParserJobs == 0)
+        finish(ParseResult::Success);
 }
 
 void CompilationDbParser::finish(ParseResult result)
 {
+    if (result != ParseResult::Failure)
+        m_guard.markAsSuccess();
+
     emit finished(result);
     deleteLater();
 }
 
-static QStringList jsonObjectFlags(const QJsonObject &object, QSet<QString> &flagsCache)
-{
-    QStringList flags;
-    const QJsonArray arguments = object["arguments"].toArray();
-    if (arguments.isEmpty()) {
-        flags = splitCommandLine(object["command"].toString(), flagsCache);
-    } else {
-        flags.reserve(arguments.size());
-        for (const QJsonValue &arg : arguments) {
-            auto flagIt = flagsCache.insert(arg.toString());
-            flags.append(*flagIt);
-        }
-    }
-
-    return flags;
-}
-
-static FilePath jsonObjectFilename(const QJsonObject &object)
-{
-    const QString workingDir = QDir::cleanPath(object["directory"].toString());
-    FilePath fileName = FilePath::fromString(QDir::cleanPath(object["file"].toString()));
-    if (fileName.toFileInfo().isRelative())
-        fileName = FilePath::fromString(QDir::cleanPath(workingDir + "/" + fileName.toString()));
-    return fileName;
-}
-
-std::vector<DbEntry> CompilationDbParser::readJsonObjects() const
-{
-    std::vector<DbEntry> result;
-
-    int objectStart = m_projectFileContents.indexOf('{');
-    int objectEnd = m_projectFileContents.indexOf('}', objectStart + 1);
-
-    QSet<QString> flagsCache;
-    while (objectStart >= 0 && objectEnd >= 0) {
-        const QJsonDocument document = QJsonDocument::fromJson(
-                    m_projectFileContents.mid(objectStart, objectEnd - objectStart + 1));
-        if (document.isNull()) {
-            // The end was found incorrectly, search for the next one.
-            objectEnd = m_projectFileContents.indexOf('}', objectEnd + 1);
-            continue;
-        }
-
-        const QJsonObject object = document.object();
-        const Utils::FilePath fileName = jsonObjectFilename(object);
-        const QStringList flags = filterFromFileName(jsonObjectFlags(object, flagsCache),
-                                                     fileName.baseName());
-        result.push_back({flags, fileName, object["directory"].toString()});
-
-        objectStart = m_projectFileContents.indexOf('{', objectEnd + 1);
-        objectEnd = m_projectFileContents.indexOf('}', objectStart + 1);
-    }
-
-    return result;
-}
-
-QStringList readExtraFiles(const QString &filePath)
-{
-    QStringList result;
-
-    QFile file(filePath);
-    if (file.open(QFile::ReadOnly)) {
-        QTextStream stream(&file);
-
-        while (!stream.atEnd()) {
-            QString line = stream.readLine();
-            line = line.trimmed();
-
-            if (line.isEmpty() || line.startsWith('#'))
-                continue;
-
-            result.push_back(line);
-        }
-    }
-
-    return result;
-}
-
-DbContents CompilationDbParser::parseProject()
-{
-    DbContents dbContents;
-    dbContents.entries = readJsonObjects();
-    dbContents.extraFileName = m_projectFilePath.toString() +
-            Constants::COMPILATIONDATABASEPROJECT_FILES_SUFFIX;
-    dbContents.extras = readExtraFiles(dbContents.extraFileName);
-    std::sort(dbContents.entries.begin(), dbContents.entries.end(),
-              [](const DbEntry &lhs, const DbEntry &rhs) {
-        return std::lexicographical_compare(lhs.flags.begin(), lhs.flags.end(),
-                                            rhs.flags.begin(), rhs.flags.end());
-    });
-    return dbContents;
-}
-
-} // namespace Internal
-} // namespace CompilationDatabaseProjectManager
+} // namespace CompilationDatabaseProjectManager::Internal
