@@ -1,33 +1,12 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of Qt Creator.
-**
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 as published by the Free Software
-** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
 
 #include "qmlprofilertextmark.h"
 
 #include "qmlprofilerconstants.h"
 #include "qmlprofilerviewmanager.h"
 #include "qmlprofilerstatisticsview.h"
+#include "qmlprofilertr.h"
 
 #include <QLabel>
 #include <QLayout>
@@ -40,7 +19,7 @@ namespace Internal {
 
 QmlProfilerTextMark::QmlProfilerTextMark(QmlProfilerViewManager *viewManager, int typeId,
                                          const FilePath &fileName, int lineNumber)
-    : TextMark(fileName, lineNumber, Constants::TEXT_MARK_CATEGORY)
+    : TextMark(fileName, lineNumber, {Tr::tr("QML Profiler"), Constants::TEXT_MARK_CATEGORY})
     , m_viewManager(viewManager)
 {
     addTypeId(typeId);
@@ -56,13 +35,6 @@ void QmlProfilerTextMark::addTypeId(int typeId)
     setLineAnnotation(statisticsView->summary(m_typeIds));
 }
 
-void QmlProfilerTextMark::clicked()
-{
-    int typeId = m_typeIds.takeFirst();
-    m_typeIds.append(typeId);
-    emit m_viewManager->typeSelected(typeId);
-}
-
 QmlProfilerTextMarkModel::QmlProfilerTextMarkModel(QObject *parent) : QObject(parent)
 {
 }
@@ -116,13 +88,13 @@ void QmlProfilerTextMarkModel::createMarks(QmlProfilerViewManager *viewManager,
 
 void QmlProfilerTextMarkModel::showTextMarks()
 {
-    for (QmlProfilerTextMark *mark : qAsConst(m_marks))
+    for (QmlProfilerTextMark *mark : std::as_const(m_marks))
         mark->setVisible(true);
 }
 
 void QmlProfilerTextMarkModel::hideTextMarks()
 {
-    for (QmlProfilerTextMark *mark : qAsConst(m_marks))
+    for (QmlProfilerTextMark *mark : std::as_const(m_marks))
         mark->setVisible(false);
 }
 
