@@ -5,6 +5,7 @@
 
 #include "syntaxhighlighter.h"
 #include "texteditorsettings.h"
+#include "syntaxhighlighterrunner.h"
 
 #include <utils/qtcassert.h>
 
@@ -73,35 +74,39 @@ const Ranges rangesForResult(
 
 }
 
-void SemanticHighlighter::incrementalApplyExtraAdditionalFormats(SyntaxHighlighter *highlighter,
-        const QFuture<HighlightingResult> &future,
-        int from, int to,
-        const QHash<int, QTextCharFormat> &kindToFormat,
-        const Splitter &splitter)
+void SemanticHighlighter::incrementalApplyExtraAdditionalFormats(
+    SyntaxHighlighterRunner *highlighter,
+    const QFuture<HighlightingResult> &future,
+    int from,
+    int to,
+    const QHash<int, QTextCharFormat> &kindToFormat,
+    const Splitter &splitter)
 {
     if (to <= from)
         return;
 
-    const int firstResultBlockNumber = int(future.resultAt(from).line) - 1;
+    const int resultStartLine = future.resultAt(from).line;
+    int formattingStartLine = 1;
 
-    // blocks between currentBlockNumber and the last block with results will
-    // be cleaned of additional extra formats if they have no results
-    int currentBlockNumber = 0;
+    // Find the line on which to start formatting, where "formatting" means to either
+    // clear out formats from outdated document versions (if there is no current result
+    // on that line), or apply the format corresponding to the respective result.
+    // Note that if there are earlier results on the same line, we have to make sure they
+    // get re-applied by adapting the from variable accordingly.
     for (int i = from - 1; i >= 0; --i) {
         const HighlightingResult &result = future.resultAt(i);
-        const int blockNumber = int(result.line) - 1;
-        if (blockNumber < firstResultBlockNumber) {
-            // stop! found where last format stopped
-            currentBlockNumber = blockNumber + 1;
-            // add previous results for the same line to avoid undoing their formats
+        if (result.line == resultStartLine) {
+            from = i;
+        } else if (result.line < resultStartLine) {
+            formattingStartLine = result.line + 1;
             from = i + 1;
             break;
         }
     }
 
     QTextDocument *doc = highlighter->document();
-    QTC_ASSERT(currentBlockNumber < doc->blockCount(), return);
-    QTextBlock currentBlock = doc->findBlockByNumber(currentBlockNumber);
+    QTC_ASSERT(formattingStartLine <= doc->blockCount(), return);
+    QTextBlock currentBlock = doc->findBlockByNumber(formattingStartLine - 1);
 
     std::map<QTextBlock, QVector<QTextLayout::FormatRange>> formatRanges;
     for (int i = from; i < to; ++i) {
@@ -109,17 +114,21 @@ void SemanticHighlighter::incrementalApplyExtraAdditionalFormats(SyntaxHighlight
             formatRanges[range.block].append(range.formatRange);
     }
 
+    QList<int> clearBlockNumberVector;
+    QMap<int, QList<QTextLayout::FormatRange>> blockNumberMap;
     for (auto &[block, ranges] : formatRanges) {
         while (currentBlock < block) {
-            highlighter->clearExtraFormats(currentBlock);
+            clearBlockNumberVector.append(currentBlock.blockNumber());
             currentBlock = currentBlock.next();
         }
-        highlighter->setExtraFormats(block, std::move(ranges));
+        blockNumberMap[block.blockNumber()] = ranges;
         currentBlock = block.next();
     }
+    highlighter->clearExtraFormats(clearBlockNumberVector);
+    highlighter->setExtraFormats(blockNumberMap);
 }
 
-void SemanticHighlighter::setExtraAdditionalFormats(SyntaxHighlighter *highlighter,
+void SemanticHighlighter::setExtraAdditionalFormats(SyntaxHighlighterRunner *highlighter,
                                                     const QList<HighlightingResult> &results,
                                                     const QHash<int, QTextCharFormat> &kindToFormat)
 {
@@ -130,20 +139,19 @@ void SemanticHighlighter::setExtraAdditionalFormats(SyntaxHighlighter *highlight
     QTextDocument *doc = highlighter->document();
     QTC_ASSERT(doc, return );
 
-    std::map<QTextBlock, QVector<QTextLayout::FormatRange>> formatRanges;
+    QMap<int, QList<QTextLayout::FormatRange>> blockNumberMap;
 
-    for (auto result : results) {
-        for (const Range &range : rangesForResult(result, doc, kindToFormat))
-            formatRanges[range.block].append(range.formatRange);
+    for (const HighlightingResult &result : results) {
+        const Ranges ranges = rangesForResult(result, doc, kindToFormat);
+        for (const Range &range : ranges)
+            blockNumberMap[range.block.blockNumber()].append(range.formatRange);
     }
 
-    for (auto &[block, ranges] : formatRanges)
-        highlighter->setExtraFormats(block, std::move(ranges));
+    highlighter->setExtraFormats(blockNumberMap);
 }
 
 void SemanticHighlighter::clearExtraAdditionalFormatsUntilEnd(
-        SyntaxHighlighter *highlighter,
-        const QFuture<HighlightingResult> &future)
+    SyntaxHighlighterRunner *highlighter, const QFuture<HighlightingResult> &future)
 {
     const QTextDocument * const doc = highlighter->document();
     QTextBlock firstBlockToClear = doc->begin();
@@ -158,6 +166,9 @@ void SemanticHighlighter::clearExtraAdditionalFormatsUntilEnd(
         }
     }
 
+    QList<int> clearBlockNumberVector;
     for (QTextBlock b = firstBlockToClear; b.isValid(); b = b.next())
-        highlighter->clearExtraFormats(b);
+        clearBlockNumberVector.append(b.blockNumber());
+
+    highlighter->clearExtraFormats(clearBlockNumberVector);
 }
